using System;
using System.Collections.Generic;
using SmallMvvm.Services.Exceptions;
using SmallMvvm.Services.Properties;

namespace SmallMvvm.Services
{
    public class ServiceProvider : IServiceProvider
    {
        private readonly Dictionary<string, object> _services = new Dictionary<string, object>();

        #region Get Service

        public object GetService(Type serviceType)
        {
            return GetService(serviceType, CreateKeyFromType(serviceType));
        }

        public object GetService(Type serviceType, string serviceName)
        {
            if (!_services.ContainsKey(serviceName))
            {
                if (serviceName == CreateKeyFromType(serviceType))
                    throw new ServiceNotAvailableException(serviceType);

                throw new ServiceNotAvailableException(serviceName, serviceType);
            }

            return _services[serviceName];
        }

        public T GetService<T>()
            where T : class
        {
            return GetService<T>(CreateKeyFromType<T>());
        }

        public T GetService<T>(string serviceName)
            where T : class
        {
            if (!_services.ContainsKey(serviceName))
            {
                if (serviceName == CreateKeyFromType<T>())
                    throw new ServiceNotAvailableException(typeof (T));

                throw new ServiceNotAvailableException(serviceName, typeof (T));
            }

            return (T) _services[serviceName];
        }

        #endregion
        
        #region Register

        public void Register<T>(T instance)
            where T : class
        {
            if (instance == null)
                throw new ArgumentNullException("instance", Resources.ServiceProvider_ServiceInstanceCannotNull);

            if (_services.ContainsKey(CreateKeyFromType<T>()))
                throw new ArgumentException(Resources.ServiceProvider_ServiceOfSubmittedTypeAlreadyExists, "instance");

            _services.Add(CreateKeyFromType<T>(), instance);
        }

        public void Register<T>(T instance, string serviceName)
            where T : class
        {
            if (instance == null)
                throw new ArgumentNullException("instance", Resources.ServiceProvider_ServiceInstanceCannotNull);

            if (serviceName == null)
                throw new ArgumentNullException("serviceName", Resources.ServiceProvider_ServiceInstanceCannotNull);

            if (serviceName.Trim() == String.Empty)
                throw new ArgumentException(Resources.ServiceProvider_ServiceNameShouldNotBeEmpty, "serviceName");

            if (serviceName.Trim() == CreateKeyFromType<T>())
                throw new ArgumentException(Resources.ServiceProvider_ServiceName_ShouldNotBeNameOfRegisteredType, "serviceName");

            if (_services.ContainsKey(serviceName.Trim()))
                throw new ArgumentException(Resources.ServiceProvider_NameAlreadyExists, "serviceName");

            _services.Add(serviceName, instance);
        }

        #endregion
        
        #region Unregister

        public void Unregister<T>()
            where T : class
        {
            Unregister<T>(CreateKeyFromType<T>());
        }

        public void Unregister<T>(string serviceName)
            where T : class
        {
            if (!_services.ContainsKey(serviceName))
                throw new ServiceNotAvailableException(serviceName, typeof(T));

            _services.Remove(serviceName);
        }

        public void UnregisterAll()
        {
            _services.Clear();
        }

        #endregion
        
        private static string CreateKeyFromType(Type type)
        {
            return type.FullName;
        }

        private static string CreateKeyFromType<T>()
        {
            return CreateKeyFromType(typeof(T));
        }
    }
}